function [preamble,payload] = LoRa_demodulator(y, SF, BW, ovs, len_preamble)
% LORA_DEMODULATOR is a simplified LoRa demodulator 
%   The demodulator assuming perfect time and frequency synchronization. 
%   Its sole purpose is to verify the the codes used for the generation 
%   of LoRa signals

% Authors: Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


chirp_duration=2^SF/BW;
fs=BW*ovs;
len=fs*chirp_duration;

t  = 0 : 1/fs : chirp_duration;
x1 = exp(1i * 2*pi * (BW/(2*chirp_duration)*t.^2 -BW/2*t));


% DEMODULATE PREAMBLE

for i=1:len_preamble+2
    start=1+(i-1)*len;
    x3 = y(start : ovs : start+len) .* conj(x1(1 : ovs : end));
    [~, idx(i)] = max(abs(fft(x3(:))));
    preamble(i)=idx(i)-1;
end


% DEMODULATE PAYLOAD

len_payload=round(((length(y)-(len_preamble+2+2.25)*len)+1)/len);

for i=1:len_payload
    start=floor((len_preamble+2+2.25)*len+(i-1)*len)-1;
    x3 = y(start : ovs : start+len) .* conj(x1(1 : ovs : end));
    [~, idx(i)] = max(abs(fft(x3(:))));
    payload(i)=idx(i)-1;
    if payload(i)==128
        payload(i)=0;
    end
end

end

