function p = compute_whitening(p)
%COMPUTE_WHITENING performs a whitening of the databytes p
%   The function performs a whitenig of the databits p. The whitening
%   sequence is computed with a LFSR with the polynomial:
%   w(x) = x^4 xor x^3 xor x^2 xor 1 and a starting value of 0x FF

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


if any(p < 0) || any(p > 255) || any(p ~= floor(p)) || ~isvector(p)
    error('p must be a vector containing integers in the uint8-range 0...255');
end


register = double(0xFF); % starting value of 0x FF

%perform whitening
for idx = 1 : length(p)
    p(idx) = bitxor(p(idx), register);
    feedback = mod(sum(dec2binvec(bitand(register, double(0xB8)))), 2);
    register = bitand(register * 2, double(0xFF)) + feedback;
end
end