function header = header_encode(len, CRC, CR)
%HEADER_ENCODE creates a LoRa-header for the input specifications
%   The function creates the header for a message with payload length len,
%   an indication of a payload CRC presence:
%   CRC = 1     payload CRC enabled
%   CRC = 0     payload CRC disabled,
%   and the code rate CR used to encode the payload.

% Authors: Holger Arthaber, Harald Eigner
%          TU Wien (EMCE), www.emce.tuwien.ac.at

% This work is licensed under a Creative Commons Attribution 4.0 International 
% (CC BY 4.0), https://creativecommons.org/licenses/by/4.0/
% You are free to:
%  - Share — copy and redistribute the material in any medium or format
%  - Adapt — remix, transform, and build upon the material for any purpose, 
%    even commercially. 
% Under the following terms:
%  - Attribution — You must give appropriate credit, provide a link to the license, 
%    and indicate if changes were made. You may do so in any reasonable manner, but not 
%    in any way that suggests the licensor endorses you or your use. 


if ~isscalar(len) || len < 0 || len > 255 || len ~= floor(len)
    error('Invalid Length');
end
if ~isscalar(CR) || CR < 5 || CR > 8 || CR ~= floor(CR)
    error('Invalid Code Rate');
end
if ~isscalar(CRC) || CRC < 0 || CRC > 1 || CRC ~= floor(CRC)
    error('Invalid Code Rate');
end

%stacking header information
header=[dec2binvec(CRC) dec2binvec((CR - 4), 3) dec2binvec(len, 8)];

%Generator matrix for the calculation of the CRC
CRC_gen(1, 1:12)=logical([1 1 1 1 0 1 0 0 1 0 0 0]);
CRC_gen(2, 1:12)=logical([1 1 1 0 1 0 1 0 0 1 0 0]);
CRC_gen(3, 1:12)=logical([0 1 0 1 1 0 0 1 0 0 1 0]);
CRC_gen(4, 1:12)=logical([1 0 0 0 0 1 1 1 0 0 0 1]);
CRC_gen(5, 1:12)=logical([0 0 0 0 0 0 0 0 1 1 1 1]);

CRC_cal = zeros(1, 5);
CRC_cal_bin = zeros(1, 5);
for idx = 1 : 5
    CRC_cal(idx) = mod(sum(header.*CRC_gen(idx, :)), 2);  %calculation of the CRC
    CRC_cal_bin(idx) = dec2binvec(CRC_cal(idx));
end

%Add header-CRC
header = fliplr(reshape(fliplr([CRC_cal_bin false false false header]), 4, 5)');
end